# Enable detailed error messages
$ErrorActionPreference = 'Stop'

try {
    # Use $PSScriptRoot to reference the script's directory
    $scriptDirectory = $PSScriptRoot

    # Define possible Python executables
    $pythonExecutables = @("python", "py")

    # Function to find a working Python executable
    function Get-PythonExecutable {
        foreach ($exe in $pythonExecutables) {
            try {
                $processInfo = New-Object System.Diagnostics.ProcessStartInfo
                $processInfo.FileName = $exe
                $processInfo.Arguments = "--version"
                $processInfo.RedirectStandardError = $true
                $processInfo.RedirectStandardOutput = $true
                $processInfo.UseShellExecute = $false
                $processInfo.CreateNoWindow = $true

                $process = New-Object System.Diagnostics.Process
                $process.StartInfo = $processInfo
                $process.Start() | Out-Null
                $process.WaitForExit()
                if ($process.ExitCode -eq 0) {
                    return $exe
                }
            } catch {
                continue
            }
        }
        throw "No working Python executable found. Please ensure that 'python' or 'py' is installed and in your PATH."
    }

    # Get a working Python executable
    $pythonExe = Get-PythonExecutable

    # Path to view_skin.py
    $viewSkinScript = Join-Path -Path $scriptDirectory -ChildPath "view_skin.py"

    # Output the paths for verification
    Write-Host "Python executable: $pythonExe"
    Write-Host "view_skin.py path: $viewSkinScript"

    # Start BHapticsSkinServer.exe in a new window
    $skinServerPath = Join-Path -Path $scriptDirectory -ChildPath "BHapticsSkinServer.exe"
    Start-Process -FilePath $skinServerPath

    # Wait a moment to ensure the server has started
    Start-Sleep -Seconds 2

    # Get list of COM port names
    $portNames = [System.IO.Ports.SerialPort]::GetPortNames()

    if (!$portNames) {
        Write-Host "No COM ports found."
        Read-Host -Prompt "Press Enter to exit"
        exit
    }

    # Create an array to hold the port information
    $ports = @()

    foreach ($portName in $portNames) {
        # Try to get the friendly name of the port
        $query = "SELECT * FROM Win32_PnPEntity WHERE Name LIKE '%($portName)%'"
        $pnpEntity = Get-CimInstance -Query $query

        if ($pnpEntity) {
            foreach ($pnp in $pnpEntity) {
                $ports += [PSCustomObject]@{
                    PortName     = $portName
                    FriendlyName = $pnp.Name
                }
            }
        } else {
            $ports += [PSCustomObject]@{
                PortName     = $portName
                FriendlyName = $portName
            }
        }
    }

    # Display the ports
    Write-Host "Available COM ports:"
    foreach ($port in $ports) {
        Write-Host "$($port.PortName) - $($port.FriendlyName)"
    }

    # Ask the user to input the COM port number directly
    $selection = Read-Host "Enter the COM port number for the RoboticSkin (e.g., '7' for 'COM7')"

    # Construct the COM port name
    $usb = "COM$selection"

    # Validate the user's input
    if ($ports.PortName -contains $usb) {
        Write-Host "Selected COM port: $usb"
    } else {
        Write-Host "Invalid selection."
        Read-Host -Prompt "Press Enter to exit"
        exit
    }

    # Set server address
    $server = "127.0.0.1"

    # Ask the user if they want to visualize the skin input
    $visualizeInput = Read-Host "Do you want to visualize the skin input? (y/n)"
    $visualizeInput = $visualizeInput.ToLower()
    if ($visualizeInput -eq 'y' -or $visualizeInput -eq '') {
        $showWindow = 'Y'
    } else {
        $showWindow = 'n'
    }

    # Ask the user if they want to record the touch
    $recordTouch = Read-Host "Do you want to record the touch? (y/n)"
    $recordTouch = $recordTouch.ToLower()

    if ($recordTouch -eq 'y' -or $recordTouch -eq '') {
        # Ask for a filename
        $filename = Read-Host "Enter filename to record interaction"
    } else {
        $filename = ""
    }

    # Ensure $filename is not null
    if ($filename -eq $null) {
        $filename = ""
    }

    # Prepare positional arguments for view_skin.py
    $positionalArguments = @()
    $positionalArguments += $server          # Server address
    $positionalArguments += $usb             # COM port
    $positionalArguments += $filename        # Filename (empty string if not recording)
    $positionalArguments += $showWindow      # 'Y' or 'n'

    # Build the command to run view_skin.py
    $processInfo = New-Object System.Diagnostics.ProcessStartInfo
    $processInfo.FileName = $pythonExe
    $processInfo.Arguments = "`"$viewSkinScript`" " + ($positionalArguments | ForEach-Object { "`"$_`"" }) -join ' '
    $processInfo.RedirectStandardError = $true
    $processInfo.RedirectStandardOutput = $true
    $processInfo.UseShellExecute = $false
    $processInfo.CreateNoWindow = $false  # Set to $true to hide the window

    # Output the command for debugging
    Write-Host "Running command:"
    Write-Host "$($processInfo.FileName) $($processInfo.Arguments)"

    $process = New-Object System.Diagnostics.Process
    $process.StartInfo = $processInfo
    $process.Start() | Out-Null

    # Read output
    $stdOut = $process.StandardOutput.ReadToEnd()
    $stdErr = $process.StandardError.ReadToEnd()
    $process.WaitForExit()

    if ($process.ExitCode -ne 0) {
        Write-Host "view_skin.py exited with code $($process.ExitCode)"
        Write-Host "Error Output:"
        Write-Host $stdErr
    } else {
        Write-Host "view_skin.py completed successfully."
        Write-Host $stdOut
    }
}
catch {
    Write-Host "An error occurred:"
    Write-Host $_.Exception.Message
    Write-Host $_.ScriptStackTrace
}
finally {
    Read-Host -Prompt "Press Enter to exit"
}
