#!/usr/bin/env python3

import serial# pip install pyserial
import numpy as np
import cv2# pip install opencv-python
import socket
import sys
import time
import json

class Skin:
    def __init__(self, name, port='/dev/ttyUSB0'):
        self.port = serial.Serial(port, 250000)
        self.name = name
        self.history = []
        self.rows,self.cols = self.getRes()
        self.frame = self.getFrame()

    def getRes(self):
      #  return 10
        while True:
            i = ord(self.port.read())
            #print(i)
            if i == 255:
                break

        count = 0
        while True:        
            i = ord(self.port.read())
            #print(i)
            if i == 255:
                break
            count += 1
            
        print("Total number of bytes: ", count)
            
        maxRow = 20
        cols = 10
            
        while cols > 1:
            if count % cols == 0:
                if count / cols > maxRow:
                    return None
                print("   %d x %d" % (count/cols, cols))
                # First found is likely best
                return (count//cols,cols) 
            cols -= 1

    def getFrame(self):
        frame = np.zeros([self.rows,self.cols])
        for r in range(self.rows):
            for c in range(self.cols):                
                i = ord(self.port.read())
                if i == 255:
                    return self.getFrame() # try again
                frame[r,c] = i
        stop = ord(self.port.read())
        if stop != 255:
            print("WARNING: the resolution seems to have changed")
            
            
        frame = frame[:,-1:0:-1]
        if self.history.__len__()<10: 
            #print(self.history)
            self.history.append(frame)
        else:
            #print(self.history)
            self.history.pop(0)
            self.history.append(frame)
            #print(self.history)
          
        avg = self.history[0]
        for f in self.history[1:]:
            avg += f
        avg /= len(self.history)
        
        #for r in range(self.rows):
            #for c in range(self.cols):   
                   #for j in range(self.history.__len__):
                        #i += self.history[j][r,c]/10 
                        
        return avg

    def refresh(self):
        self.frame = self.getFrame() / 255.
    
    def isTouched(self):
        return np.count_nonzero(self.frame > .05) > 1

    def isTouchedHeavily(self):
        return np.count_nonzero(self.frame > .5) > 5

    def show(self):
        r,c = self.frame.shape
        img = cv2.resize(self.frame, (c*40,r*20))
        #img = cv2.resize(self.frame, (self.cols*10,self.rows*20))
        cv2.imshow(self.name, img)
        return cv2.waitKey(1)

    def touchLocation(self):
        map = self.frame > .3
        return (0,0)
    
    def toBytes(self):
        #bytes = self.frame.tobytes()
        #print(self.frame.shape)
        img_bytes = cv2.resize(self.frame, (4,4))
        #img = cv2.resize(img_bytes, (self.cols*10,self.rows*20))
        #cv2.imshow("output", img)
        return bytes([ int(200.0*(i-.05)) if i>.05 else int(0) for i in img_bytes.flatten() ])


# ------------------------ HELPERS --------------------------
def hapticServerConnect(s, port=51470): 
    Host = SERVER # PC running 'BHapticsSkinServer.exe', so 'localhost' or IP address
    Port = port #BHaptics PORT 
    
    s.connect((Host, Port))
      
    print("socket connected")

def saveRecording(jsonData):
    # read file if exists
    try:
        f = open(FILENAME + ".json", "r")
        previousData = json.loads(f.read())
        f.close()
    except:
        previousData = {
            "recording": [],
        }
    
    f = open(FILENAME + ".json", "w")
    # keep trying if it is open by another program
    while True:
        try:
            f.write(json.dumps({
                "recording": previousData["recording"] + jsonData["recording"],
            }))
            break
        except:
            print("File is open by another program, trying again in 200 ms...")
            time.sleep(0.2)
    f.close()    
    print("Recording saved to file")

# ----------------------- RUN PROGRAM ------------------------
SERVER, USB, FILENAME, SHOW_WINDOW = sys.argv[1:]

hasFile = True if FILENAME else False
showWindow = True if SHOW_WINDOW != "n" and SHOW_WINDOW != "N" else False

# create a socket object
try:
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    print ("Socket created")
except socket.error as err:
    print ("socket creation error %s" %(err))

NAME = "Skin_" + USB

# Bind socket to a vest and port
skin = Skin(NAME, USB) # USB port reader for skin
hapticServerConnect(s, int(51470)) # connect to BHaptics server

print("Skin: ", skin.rows, skin.cols)

if showWindow:
    cv2.namedWindow(NAME)

# send config
config = [ 0xFF, 0xFF, 100  ] #config = [ 0xFF, 0xFF, 130  ]
config.extend( [ i for i in range(16)] )
config.append(0xFF)
s.send(bytes(config))

i = 0
isRecording = False
timestampLastTouch = 0
stopRecording = False
SECONDS_WITHOUT_TOUCH = 0.3

timeStart = 0

recordingJson = {
    "recording": [],
}

# -------------------------- LOOP ----------------------------
while True:
    i += 1

    # update skin
    skin.refresh()

    # show image
    if showWindow:
        key = skin.show()
    
    # check if program should stop
    if stopRecording or (showWindow and chr(key&255) == 'q'):
        print("Recording terminated...")
        #close file if open
        if hasFile:
            saveRecording(recordingJson)
        break
    
    # send to server
    b = skin.toBytes()
    s.send(b)
    s.send(bytes([0xFF]))

    # add to recording
    if isRecording:
        recordingJson["recording"].append({
            "data": [i for i in b],
            "timestamp": time.time(),
        })

    # write to file (when starting to record, save even if it's not touched for one second)
    if hasFile:
        if skin.isTouched():
            if not isRecording:
                print("Recording started...")
                timeStart = time.perf_counter()
                isRecording = True
                # reset recording of this side
                recordingJson["recording"] = [{
                    "data": [i for i in b],
                    "timestamp": time.time(),
                }]
            timestampLastTouch = time.time()
        elif isRecording:
            if time.time() - timestampLastTouch > SECONDS_WITHOUT_TOUCH:
                print("Total duration:", time.perf_counter() - timeStart)
                print(str(SECONDS_WITHOUT_TOUCH) + " elapsed seconds without touch, stopping recording...")
                isRecording = False
                stopRecording = True

    if i%20 != 0:
        continue

    print("Touched " if skin.isTouched() else "", "Hit hard" if skin.isTouchedHeavily() else "", skin.touchLocation())

    
